"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = require("child_process");
const index_1 = require("./index");
async function waitForClose(subprocess) {
    const timeout = setInterval(() => { }, ~0 >>> 1);
    await new Promise(resolve => {
        subprocess.once('close', () => {
            clearInterval(timeout);
            resolve();
        });
    });
}
async function waitForReady(subprocess) {
    const timeout = setInterval(() => { }, ~0 >>> 1);
    await new Promise(resolve => {
        subprocess.once('message', message => {
            if (message == 'ready') {
                clearInterval(timeout);
                resolve();
            }
        });
    });
}
const batchSize = 1000000;
async function main(args) {
    const [filename, fieldName, ...fieldNames] = args.slice(2);
    const subprocesses = {};
    const batches = { [fieldName]: [] };
    for (const name of fieldNames) {
        subprocesses[name] = child_process.fork(__filename, [filename, name]);
        batches[name] = [];
    }
    await Promise.all(Object.values(subprocesses).map(waitForReady));
    const index = new index_1.default(filename);
    await index.open();
    const cache = new Map();
    function insertBatch(name) {
        const batch = batches[name];
        batches[name] = [];
        if (batch.length)
            index.insert(batch, cache);
    }
    function sendBatch(name) {
        const batch = batches[name];
        batches[name] = [];
        if (batch.length)
            subprocesses[name].send(batch);
    }
    const handler = async (objectFields) => {
        if (objectFields == null) {
            // Insert remaining fields
            insertBatch(fieldName);
            // Send remaining batches
            for (const name in subprocesses)
                sendBatch(name);
            // Wait for other processes to close
            for (const subprocess of Object.values(subprocesses))
                subprocess.send(null);
            await Promise.all(Object.values(subprocesses).map(waitForClose));
            // Cleanup
            process.off('message', handler);
            process.once('beforeExit', async () => {
                await index.close();
            });
            return;
        }
        if (!objectFields.length)
            return;
        if (fieldNames.length) {
            for (const field of objectFields) {
                const batch = batches[field.name];
                batch.push(field);
                if (batch.length >= batchSize) {
                    if (field.name == fieldName)
                        insertBatch(field.name);
                    else
                        sendBatch(field.name);
                }
            }
        }
        else {
            // Insert immediately if possible
            if (objectFields.length >= batchSize)
                index.insert(objectFields, cache);
            else {
                const batch = batches[fieldName];
                for (const field of objectFields) {
                    batch.push(field);
                    if (batch.length >= batchSize)
                        insertBatch(fieldName);
                }
            }
        }
    };
    process.on('message', handler);
    if (process.send)
        process.send('ready');
}
main(process.argv);
//# sourceMappingURL=indexer.js.map