"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const file_1 = require("./file");
const utils_1 = require("./utils");
class JSONStore {
    constructor(filename, indent = 2) {
        this.indent = indent;
        this.file = new file_1.default(filename);
    }
    get isOpen() {
        return this.file.isOpen;
    }
    async open() {
        await this.file.open();
    }
    async close() {
        await this.file.close();
    }
    async create(objects = []) {
        const i = ' '.repeat(this.indent);
        const content = objects.length ?
            `\n${i}${objects.map(this.stringify.bind(this)).join(`,\n${i}`)}` : '';
        await this.file.open('wx');
        await this.file.write(0, `[${content}\n]\n`);
        await this.file.close();
    }
    async destroy() {
        await this.file.delete();
    }
    async lock(pos = 0, options = { exclusive: false }) {
        await this.file.lock(pos, options);
    }
    async unlock(pos = 0) {
        await this.file.unlock(pos);
    }
    async get(position) {
        const { value, start, length } = (await utils_1.readJSON(this.file.read(position)).next()).value;
        return { value: value, start, length };
    }
    async *getAll() {
        // Allow line-delimited JSON
        const firstChar = String.fromCodePoint((await this.file.read(0).next()).value[0][1]);
        const stream = this.file.read(Number(firstChar == '['));
        const jsonStream = utils_1.readJSON(stream);
        let res;
        while (!(res = await jsonStream.next()).done) {
            const result = res.value;
            yield [result.start, result.value];
        }
    }
    async getAppendPosition() {
        let first = false;
        let done = false;
        let position;
        for await (const chars of this.file.read(-1, true)) {
            for (const [i, codePoint] of chars) {
                if (codePoint == 32 /* Space */ || codePoint == 10 /* Newline */)
                    continue;
                if (!position) {
                    if (codePoint != 93 /* RightBracket */) {
                        done = true;
                        break;
                    }
                    position = i - 1;
                }
                else {
                    if (codePoint == 91 /* LeftBracket */)
                        first = true;
                    done = true;
                    break;
                }
            }
            if (done)
                break;
        }
        if (!position)
            throw new Error('Invalid JSON file');
        return { position, first };
    }
    async append(data, position) {
        const dataString = this.stringify(data);
        return await this.appendRaw(dataString, position);
    }
    async appendRaw(dataString, position, first = false) {
        if (!dataString)
            throw new Error('Cannot append empty string');
        if (!position)
            ({ position, first } = await this.getAppendPosition());
        const joiner = first ? this.joiner.slice(1) : this.joiner;
        await this.file.write(position, `${joiner}${dataString}\n]\n`);
        return {
            start: position + joiner.length,
            length: dataString.length,
            raw: dataString
        };
    }
    async set(position, value) {
        const valueString = JSON.stringify(value);
        await this.file.write(position, valueString);
    }
    async lastModified() {
        return (await this.file.stat()).mtime;
    }
    stringify(data) {
        const str = this.indent ? JSON.stringify([data], null, this.indent).slice(2 + this.indent, -2) : JSON.stringify(data);
        return str;
    }
    get joiner() {
        return `,\n${' '.repeat(this.indent)}`;
    }
}
exports.default = JSONStore;
//# sourceMappingURL=json-store.js.map